<?php
declare(strict_types=1);

namespace Pinpoint\CarouselManager\Controller\Adminhtml\Carousel;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Serialize\Serializer\Json;
use Pinpoint\CarouselManager\Helper\Config;
use Pinpoint\CarouselManager\Model\Carousel;
use Pinpoint\CarouselManager\Model\CarouselFactory;
use Pinpoint\CarouselManager\Model\CarouselRepository;
use Pinpoint\CarouselManager\Model\SlideCtaFactory;
use Pinpoint\CarouselManager\Model\SlideFactory;
use Pinpoint\CarouselManager\Model\SlideRepository;

class Save extends \Magento\Backend\App\Action
{
    protected $dataPersistor;
    /**
     * @var \Magento\Catalog\Model\ImageUploader
     */
    private $imageUploader;
    /**
     * @var Json
     */
    private $json;
    /**
     * @var Config
     */
    private $configHelper;
    /**
     * @var \Magento\Framework\UrlInterface
     */
    private $url;
    /**
     * @var CarouselFactory
     */
    private $carouselFactory;
    /**
     * @var CarouselRepository
     */
    private $carouselRepository;
    /**
     * @var SlideFactory
     */
    private $slideFactory;
    /**
     * @var SlideCtaFactory
     */
    private $slideCtaFactory;

    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Catalog\Model\ImageUploader $imageUploader,
        Json $jsonSerializer,
        Config $configHelper,
        \Magento\Framework\Url $url,
        SlideFactory $slideFactory,
        SlideCtaFactory $slideCtaFactory,
        CarouselFactory $carouselFactory,
        CarouselRepository $carouselRepository,
        \Magento\Framework\App\Request\DataPersistorInterface $dataPersistor
    ) {
        $this->dataPersistor = $dataPersistor;
        $this->imageUploader = $imageUploader;
        $this->json = $jsonSerializer;
        $this->configHelper = $configHelper;
        $this->url = $url;
        $this->carouselFactory = $carouselFactory;
        $this->slideFactory = $slideFactory;
        $this->slideCtaFactory = $slideCtaFactory;
        $this->carouselRepository = $carouselRepository;
        parent::__construct($context);
    }

    /**
     * Save action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        $data = $this->getRequest()->getPostValue();
        if ($data) {
            $id = $data['general']['carousel_id'];

            $model = $this->carouselRepository->getById($id);
            //$model = $this->_objectManager->create(\Pinpoint\CarouselManager\Model\Carousel::class)->load($id);
            if (!$model->getId() && $id) {
                $this->messageManager->addErrorMessage(__('This Carousel no longer exists.'));
                return $resultRedirect->setPath('*/*/');
            }

            if ($data['general']['carousel_id'] === '') {
                unset($data['general']['carousel_id']);
            }
            $model->setData($data['general']);
            unset($data['general']);

            try {
                $model->save();

                foreach ($data as $slideData) {
                    if (!isset($slideData['title'])) {
                        continue;
                    }

                    $slide = $this->slideFactory->create();
                    if (isset($slideData['slide_id'])) {
                        $slide->load($slideData['slide_id']);
                    }

                    $slideData = $this->_filterImageData($slideData, 'background_image');
                    $slideData = $this->_filterImageData($slideData, 'mobile_background_image');

                    if (isset($slideData['margins_and_padding'])) {
                        $slideData['margins_and_padding'] = $this->json->serialize($slideData['margins_and_padding']);
                    }

                    if (isset($slideData['mobile_margins_and_padding'])) {
                        $slideData['mobile_margins_and_padding'] = $this->json->serialize($slideData['mobile_margins_and_padding']);
                    }

                    $slide->setData($slideData);
                    $slide->setCarouselId($model->getId());
                    $slide->save();

                    for ($i = 1; $i <= $slide->getCtaNumber(); $i++) {
                        if (isset($slideData['cta_' . $i])) {
                            $ctaData = $slideData['cta_' . $i];

                            $cta = $this->slideCtaFactory->create();

                            if (isset($ctaData['slide_cta_id'])) {
                                $ctaSlideId = $ctaData['slide_cta_id'];
                                $cta->load($ctaSlideId);
                                if ($ctaData['slide_cta_id'] == '') {
                                    unset($ctaData['slide_cta_id']);
                                }
                            }

                            $cta->setData($ctaData);
                            $cta->setSortOrder($i);
                            $cta->setSlideId($slide->getId());
                            $cta->save();
                        }
                    }
                }

                $this->messageManager->addSuccessMessage(__('You saved the Carousel.'));
                $this->dataPersistor->clear('pinpoint_carouselmanager_carousel');

                if  ($this->getRequest()->getParam('back') === 'duplicate') {
                    $newCarousel = $this->copy($model);
                    $this->messageManager->addSuccessMessage(__('You duplicated the carousel.'));
                    return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $newCarousel->getId()]);
                } else if ($this->getRequest()->getParam('back') === 'close') {
                    return $resultRedirect->setPath('*/*/');
                } else if ($this->getRequest()->getParam('back') == 'preview') {
                    //Redirect to FE:
                    $url = $this->url->getUrl('pinpoint_carouselmanager/carousel/preview', ['carousel_id' => $model->getId()]);
                    return $resultRedirect->setUrl($url);
//                    return $resultRedirect->setPath('*/*/preview', ['carousel_id' => $model->getId()]);
                }

                return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $model->getId()]);

            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (\Exception $e) {
                //$this->messageManager->addExceptionMessage($e, __('Something went wrong while saving the Carousel.'));
                $this->messageManager->addErrorMessage($e->getMessage());
            }

            $this->dataPersistor->set('pinpoint_carouselmanager_carousel', $data);
            return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $this->getRequest()->getParam('carousel_id')]);
        }
        return $resultRedirect->setPath('*/*/');
    }

    protected function _filterImageData(array $data, $imageCode)
    {
        if (isset($data[$imageCode]) && is_array($data[$imageCode])) {
            if (!empty($data[$imageCode]['delete'])) {
                $data[$imageCode] = null;
            } else {
                if (isset($data[$imageCode][0]['name']) && isset($data[$imageCode][0]['tmp_name'])) {
                    $data[$imageCode] = $data[$imageCode][0]['name'];
                    $this->imageUploader->moveFileFromTmp($data[$imageCode]);
                } else {
                    unset($data[$imageCode]);
                }
            }
        } else {
            $data[$imageCode] = '';
        }

        return $data;
    }

    /**
     * @param Carousel $model
     */
    private function copy($model)
    {
        $duplicate = $this->carouselFactory->create();

        $data = $model->getData();
        unset($data['carousel_id']);

        $duplicate->setData($data);
        $duplicate->save();

        foreach ($model->getSlides() as $slide) {
            $data = $slide->getData();
            unset($data['slide_id']);

            $slideDuplicate = $this->slideFactory->create();
            $slideDuplicate->setData($data);
            $slideDuplicate->setCarouselId($duplicate->getId());
            $slideDuplicate->save();

            foreach ($slide->getCtas() as $cta) {
                $data = $cta->getData();
                unset($data['slide_cta_id']);

                $ctaDuplicate = $this->slideCtaFactory->create();
                $ctaDuplicate->setData($data);
                $ctaDuplicate->setSlideId($slideDuplicate->getId());
                $ctaDuplicate->save();
            }
        }

        return $duplicate;
    }
}
